(function(window, document) {
   /* eslint-disable strict */
   "use strict";
   /* Wrapper for the jsplayer iframe which runs in the client page
    * and propagates api calls in and analytics event out */

   var players = {}; // map element name to player wrapper API object
   var iOS = !!navigator.platform && /iPad|iPhone|iPod/.test(navigator.platform);

   if (navigator.userAgent.search("MSIE") === -1 && navigator.userAgent.search("Trident") === -1) {
      window.addEventListener("message", receiveMessage, false);
      if (iOS) {
         // Safari bug where sizes not valid when resize event arrives, apple site uses this delay
         window.addEventListener("resize", () => window.setTimeout(tellPlayersWindowSize, 500), false);
      } else {
         window.addEventListener("resize", tellPlayersWindowSize, false);
      }
   }

   function receiveMessage(event) {
      var player = players[event.data.windowName];
      if (!player || event.origin !== player.origin) return;
      if (event.data.jsplayerga) {
         if (window.ga) {
            window.ga.apply(null, event.data.jsplayerga);
         }
      } else if (event.data.jsplayerReady) {
         player.fps = event.data.fps;
         player.aspect = event.data.aspect;
         player.resolutions = event.data.resolutions;
         player.startFrame = event.data.startFrame;
         player.endFrame = event.data.endFrame;
         var splash = document.getElementById(player.elementId+"-splash");
         if (splash) splash.style.display = 'none';
         player.postMessage({command: "windowSize", width: window.innerWidth, height: window.innerHeight});
         player.ready = true;
         if (player.failedMessage) {
            player.failedMessage.remove();
            player.failedMessage = null;
         }
         if (player.options.events && player.options.events.onReady) {
            player.options.events.onReady({target: player});
         }
      } else if (event.data.jsplayerResize) {
         var r = event.data.jsplayerResize;
         if (!r.onTop) {
            player.wrapperElement.style.width = r.width + "px";
            player.wrapperElement.style.height = r.height + "px";
            if (window.blackbirdAPI.fadeElement) window.blackbirdAPI.fadeElement.style.display = "none";
         } else {
            if (!window.blackbirdAPI.fadeElement) {
               var e = document.createElement("div");
               e.id = "blackbirdAPI_fullwindow_fadeElement";
               e.style.cssText = "display: block; width: 100vw; height: 100vh; position: fixed; left: 0; top: 0; background: black; opacity: 0.7;";
               e.style.zIndex = window.blackbirdAPI.onTopZIndex - 1;
               window.blackbirdAPI.fadeElement = e;
               document.body.appendChild(e);
            }
            window.blackbirdAPI.fadeElement.style.display = "block";
         }
         player.iframe.width = r.width;
         player.iframe.height = r.height;
         player.iframe.style.position = r.onTop ? "fixed" : "static";
         player.iframe.style.left = r.onTop ? (window.innerWidth - r.width)/2 + "px" : null;
         player.iframe.style.top = r.onTop ? (window.innerHeight - r.height)/2 + "px" : null;
         player.iframe.style.zIndex = r.onTop ? window.blackbirdAPI.onTopZIndex : null;
      } else if (event.data.jsplayerZoom) {
         if (player.options.events && player.options.events.onZoom) {
            player.options.events.onZoom({target: player, zoom: event.data.jsplayerZoom});
         }
      } else if (event.data.hasOwnProperty("jsplayerSpeed")) {
         player.speed = event.data.jsplayerSpeed;
         if (player.options.events && player.options.events.onSpeedChange) {
            player.options.events.onSpeedChange({target: player, speed: event.data.jsplayerSpeed, frame: event.data.frame, fps: event.data.fps});
         }
      } else if (event.data.jsplayerSelectionChanged) {
         if (player.options.events && player.options.events.onSelectionChanged) {
            player.options.events.onSelectionChanged({target: player, selection: event.data.jsplayerSelectionChanged});
         }
      } else if (event.data.openUrl) {
         window.open(event.data.openUrl);
      } else if (event.data.clipping) {
         if (player.options.events && player.options.events.onClipping) {
            player.options.events.onClipping({target: player, clipping: event.data.clipping});
         }
      } else {
         window.console.log("Unknown player wrapper event", event, player);
      }
   }

   function tellPlayersWindowSize() {
      for (var playerName in players) {
         var player = players[playerName];
         player.postMessage({command: "windowSize", width: window.innerWidth, height: window.innerHeight});
      }
   }

   function createPlayer(elementId, options) {
      var wrapperElement = document.getElementById(elementId);
      if (!wrapperElement) {
         throw new Error("blackbirdAPI.createPlayer: element id " + elementId + " not found");
      }
      if (!options.url) {
         throw new Error("blackbirdAPI.createPlayer no url given");
      }
      var iframe = document.createElement("iframe");
      iframe.width = options.width || wrapperElement.clientWidth;
      iframe.height = options.height || wrapperElement.clientHeight;
      var playerUrl = new window.URL(options.url, window.location.href);
      var playerParams = new window.URLSearchParams(playerUrl.search);
      playerParams.set("width", iframe.width);
      playerParams.set("height", iframe.height);
      playerParams.set("wrapperOrigin", window.location.origin);
      if (options.start) playerParams.set("start", options.start);
      if (options.clip) playerParams.set("clip", ""+options.clip.inFrame+"-"+options.clip.outFrame);
      if (options.wrapperClipping) playerParams.set("wrapperClipping", true);
      playerUrl.search = playerParams.toString();
      iframe.src = playerUrl.toString();
      iframe.id = elementId;
      iframe.name = elementId;
      iframe.scrolling = "no";
      iframe.style = "border-width: 0; display: inline-block;";
      iframe.setAttribute("frameborder", "0");
      iframe.setAttribute("allowFullScreen", "true");
      iframe.setAttribute("webkitallowfullscreen", "true");
      iframe.setAttribute("mozallowfullscreen", "true");
      wrapperElement.appendChild(iframe);

      function postMessage(data) {
         iframe.contentWindow.postMessage(data, playerUrl.origin);
      }

      var playerAPI = {
         elementId: elementId,
         options: options,
         iframe: iframe,
         wrapperElement: wrapperElement,
         origin: playerUrl.origin,
         postMessage: postMessage,
         stopVideo: function() {
            postMessage({command: "stop"});
         },
         pauseVideo: function() {
            postMessage({command: "stop"});
         },
         playVideo: function() {
            postMessage({command: "play"});
         },
         seekTo: function(seconds) {
            postMessage({command: "seek", seconds: seconds});
         },
         zoom: function(width, height) {
            postMessage({command: "zoom", width: width, height: height});
         },
         speed: 0
      };
      players[elementId] = playerAPI;

      window.setTimeout(() => {
         if (playerAPI.ready) return;
         var div = document.createElement("div");
         div.style.position = "absolute";
         div.style.zIndex = window.blackbirdAPI.onTopZIndex + 2;
         div.style.height = "100%";
         div.style.width = "100%";
         div.style.fontSize = "2rem";
         var p = document.createElement("p");
         p.style.backgroundColor = "#ffff00";
         p.style.textAlign = "center";
         p.appendChild(document.createTextNode("Sorry, the blackbird player has not yet loaded properly"));
         div.appendChild(p);
         wrapperElement.prepend(div);
         playerAPI.failedMessage = div;
      }, 15000);
      return playerAPI;
   }

   if (navigator.userAgent.search("MSIE") === -1 && navigator.userAgent.search("Trident") === -1) {
      window.blackbirdAPI = {
         createPlayer: createPlayer,
         onTopZIndex: "1000000" // may be reset by client page
      };
   } else {
      window.blackbirdAPI = {
         createPlayer: function(elementId) {
            var wrapperElement = document.getElementById(elementId);
            var div = document.createElement("div");
            div.style.position = "absolute";
            div.style.zIndex = 2;
            div.style.height = "100%";
            div.style.width = "100%";
            var notSupportedP = document.createElement("p");
            notSupportedP.style.backgroundColor = "#ffff00";
            notSupportedP.appendChild(document.createTextNode("Your browser is too old to use the Blackbird Player."));
            notSupportedP.appendChild(document.createElement("br"));
            notSupportedP.appendChild(document.createTextNode("Please use Chrome, Firefox, Microsoft Edge or Safari."));
            div.appendChild(notSupportedP);
            wrapperElement.appendChild(div);
         },
         onTopZIndex: "1000000" // may be reset by client page
      };

   }

   window.onBlackbirdAPILoaded();
})(window, document); // eslint-disable-line no-undef
